@@ -1,4 +1,4 @@
-// Copyright 2020-2021 Intel Corporation
+// Copyright 2020 Intel Corporation
 // SPDX-License-Identifier: Apache-2.0
 
 #pragma once
@@ -78,7 +78,10 @@ namespace openvkl {
                              size_t index,
                              const float *p)
     {
-      volumeBuffers.makeConstant(index, {(void *)p}, VKL_DATA_SHARED_BUFFER);
+      // only used for deferred loading, which is not supported for repackNodes
+      // mode; therefore nodeIndex == leafNodeIndex.
+      volumeBuffers.makeConstant(
+          index, index, {(void *)p}, VKL_DATA_SHARED_BUFFER);
     }
 
     inline void makeConstant(VdbVolumeBuffers &volumeBuffers,
@@ -86,6 +89,7 @@ namespace openvkl {
                              const openvdb::Vec3s *p)
     {
       volumeBuffers.makeConstant(
+          index,
           index,
           {(void *)(p->asPointer() + 0),
            (void *)(p->asPointer() + 1),
@@ -254,26 +258,37 @@ namespace openvkl {
       /*
        * Load the given file.
        * If deferLeaves is true, then do not load leaf data but instead add
-       * tiles.
+       * tiles. If repackNodes is true, node data will be re-arranged for a
+       * more optimal memory layout; this option is incompatible with deferred
+       * leaf loading. Use the `usingSharedData()` method to determine if this
+       * object can be deleted after the VKLVolume is created.
        */
       OpenVdbGrid(VKLDevice device,
                   const std::string &path,
                   const std::string &field,
-                  bool deferLeaves = false);
+                  bool deferLeaves = false,
+                  bool repackNodes = true);
 
       /*
        * Load the given grid.
        * If deferLeaves is true, then do not load leaf data but instead add
-       * tiles.
+       * tiles. If repackNodes is true, node data will be re-arranged for a
+       * more optimal memory layout; this option is incompatible with deferred
+       * leaf loading. Use the `usingSharedData()` method to determine if the
+       * provided OpenVDB grid pointer (and this object, which retains a copy of
+       * the grid pointer) can be deleted after the VKLVolume is created.
        */
       OpenVdbGrid(VKLDevice device,
                   typename openvdbNativeGrid::Ptr vdb,
-                  bool deferLeaves = false);
+                  bool deferLeaves = false,
+                  bool repackNodes = true);
 
       /*
-       * To creat a VKLVolume, we simply set our parameters and commit.
+       * Create a VKLVolume.
+       * If commit is true, the volume will be committed. Otherwise, the
+       * application will need to commit the volume before use.
        */
-      VKLVolume createVolume() const;
+      VKLVolume createVolume(bool commit = true) const;
 
       /*
        * Return the number of nodes in this grid.
@@ -298,10 +313,21 @@ namespace openvkl {
        */
       void loadDeferred(size_t maxTimeMS);
 
+      /*
+       * Indicates if data provided from the OpenVDB file or OpenVDB grid
+       * pointer is being shared (without a copy made) with the created
+       * VKLVolume. If OpenVDB data is not being shared, then this object can be
+       * safely deleted after the VKLVolume is created; otherwise the OpenVDB
+       * data must be retained (either through retaining this object, or
+       * retaining the provided OpenVDB grid pointer).
+       */
+      bool usingSharedData() const;
+
       VKLDevice getVKLDevice() const;
 
      private:
       void loadTransform();
+      void loadActiveVoxelsBoundingBox();
       void loadDeferredAt(size_t i);
       void loadFromGrid(typename openvdbNativeGrid::Ptr vdb,
                         bool deferLeaves = false);
@@ -318,9 +344,15 @@ namespace openvkl {
     inline OpenVdbGrid<VdbFieldType>::OpenVdbGrid(VKLDevice device,
                                                   const std::string &path,
                                                   const std::string &field,
-                                                  bool deferLeaves)
-        : buffers(new VdbVolumeBuffers(device, vklAttributeTypes))
+                                                  bool deferLeaves,
+                                                  bool repackNodes)
+        : buffers(new VdbVolumeBuffers(device, vklAttributeTypes, repackNodes))
     {
+      if (deferLeaves && repackNodes) {
+        throw std::runtime_error(
+            "cannot enable both deferLeaves and repackNodes");
+      }
+
       openvdb::initialize();  // Must initialize first! It's ok to do this
                               // multiple times.
 
@@ -338,8 +370,6 @@ namespace openvkl {
         throw std::runtime_error(std::string("Incorrect tree type: ") +
                                  grid->type());
 
-      // Preallocate memory for all leaves; this makes loading the tree much
-      // faster.
       typename openvdbNativeGrid::Ptr vdb =
           openvdb::gridPtrCast<openvdbNativeGrid>(grid);
       loadFromGrid(vdb, deferLeaves);
@@ -347,20 +377,27 @@ namespace openvkl {
 
     template <typename VdbFieldType>
     inline OpenVdbGrid<VdbFieldType>::OpenVdbGrid(
-        VKLDevice device, typename openvdbNativeGrid::Ptr vdb, bool deferLeaves)
-        : buffers(new VdbVolumeBuffers(device, vklAttributeTypes))
+        VKLDevice device,
+        typename openvdbNativeGrid::Ptr vdb,
+        bool deferLeaves,
+        bool repackNodes)
+        : buffers(new VdbVolumeBuffers(device, vklAttributeTypes, repackNodes))
     {
+      if (deferLeaves && repackNodes) {
+        throw std::runtime_error(
+            "cannot enable both deferLeaves and repackNodes");
+      }
+
       openvdb::initialize();
       grid = vdb;
       loadFromGrid(vdb, deferLeaves);
     }
 
     template <typename VdbFieldType>
-    inline VKLVolume OpenVdbGrid<VdbFieldType>::createVolume(
-) const
+    inline VKLVolume OpenVdbGrid<VdbFieldType>::createVolume(bool commit) const
     {
       assert(buffers);
-      return buffers->createVolume();
+      return buffers->createVolume(commit);
     }
 
     template <typename VdbFieldType>
@@ -423,6 +460,16 @@ namespace openvkl {
       }
     }
 
+    template <typename VdbFieldType>
+    inline bool OpenVdbGrid<VdbFieldType>::usingSharedData() const
+    {
+      if (buffers) {
+        return buffers->usingSharedData();
+      } else {
+        return false;
+      }
+    }
+
     template <typename VdbFieldType>
     inline VKLDevice OpenVdbGrid<VdbFieldType>::getVKLDevice() const
     {
@@ -459,6 +506,21 @@ namespace openvkl {
                                 i2o[14]);
     }
 
+    template <typename VdbFieldType>
+    inline void OpenVdbGrid<VdbFieldType>::loadActiveVoxelsBoundingBox()
+    {
+      assert(grid);
+      const auto &activeVoxelBoundingBox = grid->evalActiveVoxelBoundingBox();
+
+      buffers->setActiveVoxelsBoundingBox(
+          box3i(vec3i(activeVoxelBoundingBox.min().x(),
+                      activeVoxelBoundingBox.min().y(),
+                      activeVoxelBoundingBox.min().z()),
+                vec3i(activeVoxelBoundingBox.max().x(),
+                      activeVoxelBoundingBox.max().y(),
+                      activeVoxelBoundingBox.max().z())));
+    }
+
     template <typename VdbFieldType>
     inline void OpenVdbGrid<VdbFieldType>::loadDeferredAt(size_t i)
     {
@@ -479,14 +541,18 @@ namespace openvkl {
     inline void OpenVdbGrid<VdbFieldType>::loadFromGrid(
         typename openvdbNativeGrid::Ptr vdb, bool deferLeaves)
     {
-      const size_t numTiles  = vdb->tree().activeTileCount();
+      // Preallocate memory for all nodes; this makes loading the tree much
+      // faster.
       const size_t numLeaves = vdb->tree().leafCount();
-      buffers->reserve(numTiles + numLeaves);
+      const size_t numTiles  = vdb->tree().activeTileCount();
+      buffers->reserve(numLeaves, numTiles);
       if (deferLeaves)
         deferred.reserve(numLeaves);
 
       loadTransform();
 
+      loadActiveVoxelsBoundingBox();
+
       const auto &root = vdb->tree().root();
       for (auto it = root.cbeginChildOn(); it; ++it)
         Builder<typename openvdbTree::RootNodeType::ChildNodeType>::visit(
